/*
 * Type representations for interfaces.
 *
 * Copyright (C) 2019, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

/* General capability type. */

#define L4_CAP_TYPE             "l4_cap_idx_t"

/* A flexpage type that can hold extra information (such as the "hot spot"). */

#define L4_FPAGE_TYPE           "l4_snd_fpage_t"

/* IPC opcode type. */

#define L4_OPCODE_TYPE          "l4_uint32_t"
#define L4_OPCODE_PADDING_TYPE  "l4_uint32_t"

/* Specifier values. */

enum specifier
{
  IN_PARAMETER = 1,
  OUT_PARAMETER = 2,
  INOUT_PARAMETER = 3,
  ANY_PARAMETER = 3,
};

/* Data class of a parameter. */

enum parameter_class
{
  WORD_CLASS = 1,
  CAP_ITEM = 2,
  FPAGE_ITEM = 4,
  ITEM_CLASS = 6, /* all item values combined */
  ANY_CLASS = 7,  /* WORD_CLASS | ITEM_CLASS */
};

/* Component roles. */

enum component_role
{
  CLIENT_ROLE,
  SERVER_ROLE,
};

/* Function roles. */

enum function_role
{
  COMPLETION_ROLE,
  GENERAL_FUNCTION_ROLE,
};

/* Role of a parameter in output. */

enum parameter_role
{
  SIGNATURE_ROLE,
  STRUCTURE_ROLE,
  INVOCATION_ROLE,
};

/* Role of a name or signature. */

enum signature_role
{
  ACCESS_ROLE = 1,
  DECLARATION_ROLE = 2,
  DEFINITION_ROLE = 4,
  MEMBER_ROLE = 8,
  MEMBER_DECLARATION_ROLE = 10, /* DECLARATION_ROLE | MEMBER_ROLE */
};

/* Output language. */

enum language
{
  C_LANGUAGE,
  CPP_LANGUAGE,
};

/* Include declarations. */

struct include
{
  char *filename;
  struct include *tail;
};

/* An identifier with a link to other identifiers. */

struct identifier
{
  char *identifier;
  struct identifier *tail;
};

/* A single parameter description linking to other parameters. */

struct parameter
{
  enum specifier specifier;
  enum parameter_class cls;
  struct identifier *identifiers;
  struct parameter *tail;
};

/* A single signature description linking to other signatures. */

struct signature
{
  char *qualifier;
  char *operation;
  struct parameter *parameters;
  struct attribute *attributes;
  struct signature *tail;
};

/* A single attribute description linking to other attributes. */

struct attribute
{
  char *attribute;
  struct identifier *identifiers;
  struct attribute *tail;
};

/* A single interface description linking to other interfaces. */

struct interface
{
  char *name;
  struct signature *signatures;
  struct attribute *attributes;
  struct include *includes;
  struct interface *tail;
};
